; 04OCT12 V00.00 PJS New
; 10OCT12 V00.01 PJS added FAN PWM output

;****************************************************************************************
; port list
; Vdd(1)
; RA5(2) down switch
; RA4(3) up switch
; RA3(4) (mclr)
; RA2(5) PWM out
; RA1(6) PGCLK strip 5 (fan on)
; RA0(7) PGDAT strip 4 (fan PWM)
; Vss(8)
;****************************************************************************************


    list	p=PIC12F1501, f=INHX8M ; set processor type and output file type (intel merged hex (.hex))
    ; PIC12F1501 Configuration Bit Settings
    #include "p12F1501.inc"
    __CONFIG _CONFIG1, _FOSC_INTOSC & _WDTE_OFF & _PWRTE_ON & _MCLRE_OFF & _CP_OFF & _BOREN_ON & _CLKOUTEN_OFF
    __CONFIG _CONFIG2, _WRT_OFF & _STVREN_ON & _BORV_LO & _LPBOR_ON & _LVP_OFF

;Constants
#define	CODE_VER_STRING "Peter Shabino 10OCT13 code for 10W led driver card V00.01 www.wire2wire.org" ;Just in ROM !!!
FANLOW		equ		0x40	; PWM setting for fan low speed
FANHIGH		equ		0x70	; PWM setting for fan high speed

;Vars (0x20 - 0x4F)
temp        equ     0x20
buttons     equ     0x21
count       equ     0x22
delay0      equ     0x23


;Global (0x70-0x7F) accessable in all banks
level       equ     0x70




;put the following at address 0000h
	org     0000h
	goto    START				;vector to initialization sequence

;put the following at address 0004h
	org     0004h
	goto    ENDLOOP				; no IRQs so far

;put the following at address 0005h
	org     0005h

;******************************************************************
;set up registers
START
;******************************************************************
    clrf    BSR                 ; switch bank 0
    clrf    PORTA				; set all IOs low
	;------------------
	incf    BSR, F              ; switch bank 1
	;------------------
    movlw   0x78                ; set up oscilator to 16MHz
    movwf   OSCCON
WAITOSCREADY                    ; check if the oscilator is ready
    btfss   OSCSTAT, 4
    goto    WAITOSCREADY
WAITOSCSTABLE                   ; check if the oscilator is stable
    btfss   OSCSTAT, 0
    goto    WAITOSCREADY
    movlw   0xF8				; set up IOs PWM out, fan on, and fan pwm as outputs reset inputs
    movwf   TRISA
    movlw   0xDF				; Set up the option reg Weak pull up enabled, int rising edge, TMR0 on internal clock, inc on high to low, prescaller not assigned to TMR0, prescalar = 1:256
    movwf   OPTION_REG
	;------------------
	incf    BSR, F              ; switch bank 3
    incf    BSR, F
	;------------------
    clrf    ANSELA				; Set all IOs to digital inputs (disable analog)
	;------------------
    movlw   0x0C                ; switch bank 12
    movwf   BSR
	;------------------
    movlw   0x0F				; Set PWM to 0%
    movwf   PWM1DCL
    movwf   PWM2DCL
    movlw   0x00				; Set PWM to 0%
    movwf   PWM1DCH
    movwf   PWM2DCH
    movlw   0xC0				; enable PWM1, Output to PWM1 pin, PWM active high
    movwf   PWM1CON
    movwf   PWM2CON
	;------------------
    clrf    BSR					; bank 0
	;------------------
    movlw   0x00				; set inital level to 0
    movwf   level
    clrf    TMR2				; clear timer2
    movlw   0xFF				; set the period register to FF (max period)
    movwf   PR2
    movlw   0x07				; 1:1 post scalar, enable timer, prescaler 1:64
    movwf   T2CON


;******************************************************************
; Main code loop
MAINLOOP
;******************************************************************
	; first thing wait for both buttons to be up (=1)
	call	_DEBOUNCE			; call button debounce code and get new value
    movf    buttons, W			; move button value to W
    xorlw   0x30				; Xor button up value with W (0x30 = both buttons up)
    btfss   STATUS, Z			; check if result is zero if so skip next instruction
    goto    MAINLOOP			; go back to start of code and wait for buttons to be up

	; now wait for one or more buttons to be pressed.
BUTTONSUP
	call	_DEBOUNCE			; call button debounce code and get new value
    movf    buttons, W			; move button value to W
    xorlw   0x30				; Xor button up value with W (0x30 = both buttons up)
    btfsc   STATUS, Z			; check if result is NOT zero if so skip next instruction
    goto    BUTTONSUP			; go back to BUTTONSUP and wait for button to be pressed

    ; at this point a valid button press has been detected. buttons will be one of these values 0x00 (both), 0x20 (up), or 0x10 (down)

	; both buttons
	movf	buttons, W			; grab the value of buttons
	btfss	STATUS, Z			; if the value was 0 skip goto
	goto	UPBUTTON			; not both button skip to the next check
	movf	level, W			; grab the value of level
	btfss	STATUS, Z			; if the value was 0 skip goto
	goto	BOTHOFF
	; Lamp is already off so turn it on to full brightness
	movlw	0x0A				; set level value to 10 (full on)
	movwf	level
	goto	SETPWM				; goto code for setting the PWM value based off level setting
BOTHOFF
	; Lamp is already on so turn it off
	clrf	level				; set level value to 0 (full off)
	goto	SETPWM				; goto code for setting the PWM value based off level setting

	; up button
UPBUTTON
    movf    buttons, W			; grab the value of buttons
    xorlw   0x20				; Xor 0x20 (up button) with button value
    btfss   STATUS, Z			; check if result is 0 if so skip goto 
    goto    DOWNBUTTON
	movlw	0x0A				; set W to max value (10)
	subwf	level, W			; subtract W from F (store in W)
    btfsc   STATUS, C			; if W > f(level) then skip goto
    goto    MAINLOOP			; if already at max brightness (W = f) then goto mainloop
    incf    level, F			; add one to the level value
	goto	SETPWM				; goto code for setting the PWM value based off level setting

    ; down  button
DOWNBUTTON
	movf    buttons, W			; grab the value of buttons
    xorlw   0x10				; Xor 0x10 (down button) with button value
    btfss   STATUS, Z			; check if result is 0 if so skip goto
    goto    MAINLOOP			; catch all should never hit this
	movf	level, W			; move level to W
	btfsc	STATUS, Z			; if level is 0 skip goto
	goto	MAINLOOP			; if level is already 0 stop and return to mainloop
    decf    level, F
	goto	SETPWM				; goto code for setting the PWM value based off level setting
    goto    MAINLOOP

SETPWM
	movf	level, W			; grab the value of level and move to W
	btfsc	STATUS, Z			; check if the value is 0 skip if not
	bcf		PORTA, 1			; disable fan if used
	btfss	STATUS, Z			; check if the value is 0 skip if 0
	bsf		PORTA, 1			; enable fan if used (turn on the fan if LED is on at all)
	;------------------
    movlw	0x0C                ; switch bank 12
    movwf	BSR
	;------------------
	btfss	PWM1CON, 6			; check if the PWM engine is shut down if so enable it
	bsf		PWM1CON, 6			; enable engine
	movf	level, W			; grab the value of level and move to W
	btfss	STATUS, Z			; check if the value is 0
	goto	SETPWM1				; level does not match go to next check
	clrf	PWM1DCL				; Set PWM to 0%
    clrf	PWM1DCH
	clrf	PWM2DCL				; Set FAN PWM to 0%
    clrf	PWM2DCH
	goto	SETPWMDONE			; done setting PWM
SETPWM1
    movlw   0xFF				; Set FAN PWM to low
    movwf   PWM2DCL
    movlw   FANLOW
    movwf   PWM2DCH
	movf	level, W			; grab the value of level and move to W
	xorlw	0x01				; Xor level value with this setting
	btfss	STATUS, Z			; check if the result is 0
	goto	SETPWM2				; level does not match go to next check
    movlw   0xFF				; Set PWM to 10%
    movwf   PWM1DCL
    movlw   0x1A
    movwf   PWM1DCH
	goto	SETPWMDONE			; done setting PWM
SETPWM2
	movf	level, W			; grab the value of level and move to W
	xorlw	0x02				; Xor level value with this setting
	btfss	STATUS, Z			; check if the result is 0
	goto	SETPWM3				; level does not match go to next check
    movlw   0xFF				; Set PWM to 20%
    movwf   PWM1DCL
    movlw   0x34
    movwf   PWM1DCH
	goto	SETPWMDONE			; done setting PWM
SETPWM3
	movf	level, W			; grab the value of level and move to W
	xorlw	0x03				; Xor level value with this setting
	btfss	STATUS, Z			; check if the result is 0
	goto	SETPWM4				; level does not match go to next check
    movlw   0xFF				; Set PWM to 30%
    movwf   PWM1DCL
    movlw   0x4E
    movwf   PWM1DCH
	goto	SETPWMDONE			; done setting PWM
SETPWM4
	movf	level, W			; grab the value of level and move to W
	xorlw	0x04				; Xor level value with this setting
	btfss	STATUS, Z			; check if the result is 0
	goto	SETPWM5				; level does not match go to next check
    movlw   0xFF				; Set PWM to 40%
    movwf   PWM1DCL
    movlw   0x68
    movwf   PWM1DCH
	goto	SETPWMDONE			; done setting PWM
SETPWM5
    movlw   0xFF				; Set FAN PWM to high
    movwf   PWM2DCL
    movlw   FANHIGH
    movwf   PWM2DCH
	movf	level, W			; grab the value of level and move to W
	xorlw	0x05				; Xor level value with this setting
	btfss	STATUS, Z			; check if the result is 0
	goto	SETPWM6				; level does not match go to next check
    movlw   0xFF				; Set PWM to 50%
    movwf   PWM1DCL
    movlw   0x82
    movwf   PWM1DCH
	goto	SETPWMDONE			; done setting PWM
SETPWM6
	movf	level, W			; grab the value of level and move to W
	xorlw	0x06				; Xor level value with this setting
	btfss	STATUS, Z			; check if the result is 0
	goto	SETPWM7				; level does not match go to next check
    movlw   0xFF				; Set PWM to 60%
    movwf   PWM1DCL
    movlw   0x9C
    movwf   PWM1DCH
	goto	SETPWMDONE			; done setting PWM
SETPWM7
	movf	level, W			; grab the value of level and move to W
	xorlw	0x07				; Xor level value with this setting
	btfss	STATUS, Z			; check if the result is 0
	goto	SETPWM8				; level does not match go to next check
    movlw   0xFF				; Set PWM to 70%
    movwf   PWM1DCL
    movlw   0xB6
    movwf   PWM1DCH
	goto	SETPWMDONE			; done setting PWM
SETPWM8
	movf	level, W			; grab the value of level and move to W
	xorlw	0x08				; Xor level value with this setting
	btfss	STATUS, Z			; check if the result is 0
	goto	SETPWM9				; level does not match go to next check
    movlw   0xFF				; Set PWM to 80%
    movwf   PWM1DCL
    movlw   0xD0
    movwf   PWM1DCH
	goto	SETPWMDONE			; done setting PWM
SETPWM9
	movf	level, W			; grab the value of level and move to W
	xorlw	0x09				; Xor level value with this setting
	btfss	STATUS, Z			; check if the result is 0
	goto	SETPWM10			; level does not match go to next check
    movlw   0xFF				; Set PWM to 90%
    movwf   PWM1DCL
    movlw   0xEA
    movwf   PWM1DCH
	goto	SETPWMDONE			; done setting PWM
SETPWM10
	movf	level, W			; grab the value of level and move to W
	xorlw	0x0A				; Xor level value with this setting
	btfss	STATUS, Z			; check if the result is 0
	goto	SETPWM11			; level does not match go to next check
    movlw   0xFF				; Set PWM to 100%
    movwf   PWM1DCL
    movwf   PWM1DCH
	; Due to the way the PWM engine works there is a small timing missmatch that results in less than 100% PWM cycle.
	; to work around this disable the PWM engine and set the output pin high
	bcf		PWM1CON, 6			; disable the PWM engine
	goto	SETPWMDONE			; done setting PWM
SETPWM11
	clrf	PWM1DCL				; catch all Set PWM to 0%
    clrf	PWM1DCH
SETPWMDONE
	;------------------
    clrf    BSR					; bank 0
	;------------------
	bsf		PORTA, 2			; set RA2 high for case where PWM is disabled for 100% brightness
    goto    MAINLOOP


ENDLOOP
    goto    ENDLOOP

;###############################################################################
; Functions
;###############################################################################

;****************************************
; read and debounce all the button inputs
_DEBOUNCE
;****************************************
    call    _DELAY0				; slow down the sample rate
    movf    PORTA, W			; grab the button inputs
    andlw   0x30				; mask off all bits other than the buttons
    movwf   temp				; save a temp copy
    xorwf   buttons, W			; check if this value matches the previous value
    btfsc   STATUS, Z			; if it does not match skip goto
    goto    BUTTONMATCH
    clrf    count				; if new value detected clear the counter
    movf    temp, W				; grab new value in temp
    movwf   buttons				; store it as buttons
BUTTONMATCH
    incfsz  count, F			; add one to count if 256 samples (0xFF -> 0x00 rollover) exit function
    goto    _DEBOUNCE			; if under go to start of function
	return						; function done return


;**************************************** 
; delay loop 0
_DELAY0                         ;2
;****************************************
	MOVLW   0xff                ;1		Set up inital value
	MOVWF   delay0              ;1		Store it
DELAY_LOOP0
   	DECFSZ  delay0, F           ;1(2)	subtract 1 from delay and skip next instruction if result is 0x00
	GOTO    DELAY_LOOP0         ;2		if not go to start of function
	RETURN                      ;2		function done return




	de  CODE_VER_STRING			; store description string at the end of the code
	end							; end of program

